using System;
using com.agilefaqs.rft.domain;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions.strategies
{
    [TestFixture]
    public class LastNameInitialStrategyTest
    {
        [SetUp]
        public void SetUp()
        {
            m = new MockRepository();
            DomainNameService dns = m.Stub<DomainNameService>();
            suggestionUtil = m.StrictMock<SuggestionsUtil>(dns);
            userService = m.StrictMock<UserService>();
            lastNameInitialYearStrategy = new LastNameInitialStrategy(userService, suggestionUtil);
        }

        private MockRepository m;
        private const String COUNTRY_MONIKER = "Indian";
        private const String FIRST_NAME = "Naresh";
        private const String LAST_NAME = "Jain";

        private static readonly String LAST_NAME_INTIAL = Convert.ToString(LAST_NAME[0]);

        private readonly ChildStrategyParam childStrategyParam =
            new ChildStrategyParam(FIRST_NAME, COUNTRY_MONIKER, LAST_NAME);


        private readonly String Identity = FIRST_NAME + LAST_NAME_INTIAL + "@" + COUNTRY_MONIKER + ".com";

        private SuggestionsUtil suggestionUtil;

        private UserService userService;


        private LastNameInitialStrategy lastNameInitialYearStrategy;

        [Test]
        public virtual void shouldGenerateSuggestionBasedOnLastNameInitialStratey()
        {
            String token1 = childStrategyParam.Token1 + (LAST_NAME_INTIAL);

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(suggestionUtil.appendTokensForId(token1, token2)).Return(Identity);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(true);
            stopRecordingAndStartReplay();
            String Id = lastNameInitialYearStrategy.generateId(childStrategyParam);
            Assert.AreEqual(Id, Identity);
        }

        [Test]
        public virtual void shouldReturnNullIfIdIsNotAvailable()
        {
            String token1 = childStrategyParam.Token1 + (LAST_NAME_INTIAL);

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(null);
            stopRecordingAndStartReplay();
            String generatedId = lastNameInitialYearStrategy.generateId(childStrategyParam);
            Assert.IsNull(generatedId);
        }

        [Test]
        public virtual void shouldReturnNullIfIdTokenReturnedBySuggestionUtilIsNullBasedOnYearStratey()
        {
            String token1 = childStrategyParam.Token1 + (LAST_NAME_INTIAL);

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(suggestionUtil.appendTokensForId(token1, token2)).Return(Identity);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(false);
            stopRecordingAndStartReplay();
            String generatedId = lastNameInitialYearStrategy.generateId(childStrategyParam);
            Assert.IsNull(generatedId);
        }

        [TearDown]
        public virtual void verifyMocks()
        {
            m.VerifyAll();
        }

        private void stopRecordingAndStartReplay()
        {
            m.ReplayAll();
        }
    }
}