using System;
using com.agilefaqs.rft.domain;
using NUnit.Framework;
using Rhino.Mocks;

namespace com.agilefaqs.rft.suggestions.strategies
{
    [TestFixture]
    public class SimpleTokenStrategyTest
    {
        [SetUp]
        public void SetUp()
        {
            m = new MockRepository();
            DomainNameService dns = m.Stub<DomainNameService>();
            suggestionUtil = m.StrictMock<SuggestionsUtil>(dns);
            userService = m.StrictMock<UserService>();
            simpleTokenStrategy = new SimpleTokenStrategy(userService, suggestionUtil);
        }

        private MockRepository m;


        private const String FIRST_NAME = "Naresh";
        private const String LAST_NAME = "Jain";

        private readonly ChildStrategyParam childStrategyParam = new ChildStrategyParam(FIRST_NAME, LAST_NAME, null);


        private readonly String Identity =
            String.Concat(String.Concat(String.Concat(FIRST_NAME, "@"), LAST_NAME), ".com");

        private SuggestionsUtil suggestionUtil;

        private UserService userService;


        private SimpleTokenStrategy simpleTokenStrategy;


        [Test]
        public virtual void shouldGenerateSuggestionBasedOnYearStratey()
        {
            String token1 = childStrategyParam.Token1;

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(suggestionUtil.appendTokensForId(token1, token2)).Return(Identity);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(true);
            stopRecordingAndStartReplay();
            String Id = simpleTokenStrategy.generateId(childStrategyParam);
            Assert.AreEqual(Id, Identity);
        }

        [Test]
        public virtual void shouldReturnNullIfIdIsNotAvailable()
        {
            String token1 = childStrategyParam.Token1;

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(null);
            stopRecordingAndStartReplay();
            String generatedId = simpleTokenStrategy.generateId(childStrategyParam);
            Assert.IsNull(generatedId);
        }

        [Test]
        public virtual void shouldReturnNullIfIdTokenReturnedBySuggestionUtilIsNullBasedOnYearStratey()
        {
            String token1 = childStrategyParam.Token1;

            String token2 = childStrategyParam.Token2;
            Expect.Call(suggestionUtil.getIdentityTokens(token1, token2)).Return(new IDTokens(token1, token2));
            Expect.Call(suggestionUtil.appendTokensForId(token1, token2)).Return(Identity);
            Expect.Call(userService.isIdentityAvailable(Identity)).Return(false);
            stopRecordingAndStartReplay();
            String generatedId = simpleTokenStrategy.generateId(childStrategyParam);
            Assert.IsNull(generatedId);
        }

        [TearDown]
        public virtual void verifyMocks()
        {
            m.VerifyAll();
        }

        private void stopRecordingAndStartReplay()
        {
            m.ReplayAll();
        }
    }
}