using System.Collections.Generic;
using com.agilefaqs.rft.domain;

namespace com.agilefaqs.rft.suggestions.strategies
{
    public class ChildSuggestionFactory
    {
        private readonly List<ChildSuggestionStrategy> strategies = new List<ChildSuggestionStrategy>();
        private readonly List<ChildSuggestionStrategy> strategiesForSuggestionOne = new List<ChildSuggestionStrategy>();

        public ChildSuggestionFactory()
        {
            
        }
        public ChildSuggestionFactory(UserService userService, SuggestionsUtil utils,
                                      RandomNumberGenerator randomNumberGenerator)
        {
            SimpleTokenStrategy simpleTokenStrategy = new SimpleTokenStrategy(userService, utils);
            YearBasedStrategy yearBasedStrategy = new YearBasedStrategy(userService, utils);
            RandomNumberBasedStrategy randomNumberBasedStrategy =
                new RandomNumberBasedStrategy(userService, utils, randomNumberGenerator);
            strategies.Add(simpleTokenStrategy);
            strategies.Add(new FirstNameAppendLastNameStrategy(userService, utils));
            strategies.Add(new LastNameInitialStrategy(userService, utils));
            strategies.Add(yearBasedStrategy);
            strategies.Add(new LastNameInitialYearBasedStrategy(userService, utils));
            strategies.Add(randomNumberBasedStrategy);

            strategiesForSuggestionOne.Add(simpleTokenStrategy);
            strategiesForSuggestionOne.Add(yearBasedStrategy);
            strategiesForSuggestionOne.Add(randomNumberBasedStrategy);
        }

        public virtual List<ChildSuggestionStrategy> getStrategies()
        {
            return strategies;
        }

        public virtual List<ChildSuggestionStrategy> getChildStrategiesForSuggestionOne()
        {
            return strategiesForSuggestionOne;
        }
    }
}